;;; -*- Mode: LISP; Syntax: Common-lisp; Package: TMS -*-

(in-package 'tms)

;;;; Extras for Assumption-based TMS

(export '(env-difference assumption-difference show-e-diffs
	  get-contradiction-statistics parse-contradiction-file
	  put-asn-prop get-asn-prop
	  write-contradiction-statistics env-context))

(defun put-asn-prop (asn value pname)
  (setf (getf (assumption-datum asn) pname) value))

(defun get-asn-prop (asn pname)
  (getf (assumption-datum asn) pname))

(defun env-difference (e1 e2)
  (multiple-value-bind (e1-e2 e2-e1)
      (assumption-difference (env-assumptions e1) (env-assumptions e2))
    (values (make-an-env e1-e2)
	    (make-an-env e2-e1))))

(defun env-orderp (e1 e2) (< (tms::env-unique e1) (tms::env-unique e2)))

(defun assumption-difference (as1 as2 &aux e1-e2 e2-e1)
  ;assumes list of assumptions is ordered, returns sorted lists.
  (do ((as1 as1)
       (as2 as2)
       (done? nil))
      (done? (values (nreverse e1-e2) (nreverse e2-e1)))
    (cond (as1
	   (cond (as2
		  (cond ((eq (car as1) (car as2))
			 (setq as1 (cdr as1) as2 (cdr as2)))
			((assumption-orderp (car as1) (car as2))
			 (push (car as1) e1-e2)
			 (setq as1 (cdr as1)))
			(t (push (car as2) e2-e1)
			   (setq as2 (cdr as2)))))
		 (t (setq e1-e2 (nreconc as1 e1-e2))
		    (setq done? t))))
	  (t (setq e2-e1 (nreconc as2 e2-e1))
	     (setq done? t)))))

(defun show-e-diffs (e1 e2)
  (multiple-value-bind (e1-e2 e2-e1)
      (assumption-difference (env-assumptions e1) (env-assumptions e2))
    (format t "~%In E1-E2:")
    (if e1-e2 (dolist (a e1-e2)
		(format t "~%   ~A" (adb::lprinc-string (adb::view-assumption a))))
	(format t "~%   none."))
    (format t "~%In E2-E1:")
    (if e2-e1 (dolist (a e2-e1)
		(format t "~%   ~A" (adb::lprinc-string (adb::view-assumption a))))
	(format t "~%   none."))))

;;;; More debugging stuff

(defun env-context (e &optional (break? nil))
  (format t "~%  ~D consequences of ~A" (length (tms:env-nodes e)) e)
  (dolist (n (tms:env-nodes e))
    (adb::lprint (adb::view-node n))
    (if break? (read-char))))

(defun string-assumption (assumption)
  (or (assumption-string assumption)
      (setf (assumption-string assumption)
	    (compute-assumption-string assumption))))

(defun compute-assumption-string (asn)
  (let ((node (get-assumption-node asn))
	(name nil))
    (cond ((careful-node? node) ;; Moral equivalent of (typep node 'node)
	   (setq name (adb::view-node node)))
	  (t (error "~% Can't find node name for ~A" asn) (setq name '("TOTAL" . "LOSER"))))
    (format nil "~A=~A" (car name) (cdr name))))

(defun fix-goddam-assumptions ()
  (dolist (asn *assumptions*)
    (setf (assumption-string asn) (compute-assumption-string asn))))

(defun find-assumption-for (node)
  ;; Goes down the justifications until one whose informant is TMS:ASSUME
  ;; is found.
  (dolist (j (node-justifications node))
    (if (eq (car j) 'assume) (return (cadr j)))))

(defun subsumed-envs? (subset set)
  (every #'(lambda (e1) (some #'(lambda (e2) (subsumed-env? e1 e2)) set))
	 subset))

(defun subsumed-env? (e1 e2)
  (or (eq e1 e2) (eq (tms:compare-env e1 e2) 'tms:subset21)))

