;;; -*- Mode: LISP; Syntax: Common-lisp; Package: QPE -*-

(defvar *axiom-indicators* '(rassert! rjustify rnogood rassume nogood justify assume assert!
				      *QC-Qstate* install-derived-qstate-pairs
				      adb:add-to-nested-alist))


(defun axioms-in-form (form)
  (cond ((null form) 0)
	((symbolp form)
	 (if (member form *axiom-indicators*) 1 0))
	((not (listp form)) 0)
	(t (+ (axioms-in-form (car form))
	      (axioms-in-form (cdr form))))))

(setq foo '(defProcess (Liquid-Flow-Thermal ?lf)
   Individuals ((?src :type Thermal-Contained-Liquid)
		(?dst :type Thermal-Contained-Liquid)
		(?lf :conditions ((process-instance liquid-flow) ?lf)
		                 (?lf src ?src)
		                 (?lf dst ?dst)))
   QuantityConditions ((Active ?lf)
		       (not (equal-to (A (Temperature ?src)) (A (Temperature ?dst)))))
   Relations ((Quantity Thermal-Mixing)	;want gas & liquid to coexist
	      (Q= Thermal-Mixing (- (Temperature ?src) (Temperature ?dst))))
   Influences ((I+ (Intensive-Energy ?dst) (A Thermal-Mixing)))))

(defun axioms-in-file (from-file)
  (let ((count 0) (marker (cons nil nil)))
  (with-open-file (in-box from-file :direction :input)
    (do ((form (read in-box nil marker) (read in-box nil marker)))
	((eq form marker) count)
      (incf count (axioms-in-form (macroexpand form)))))))

(defun tabulate-domain (dfiles)
  (let ((total 0) (fcount 0))
    (dolist (dfile dfiles)
      (setq fcount (axioms-in-file dfile))
      (incf total fcount)
      (format t "~% ~D in ~A." fcount dfile))
    (format t "~%~D axiom-equivalents total." total)))

(setq *steam-plant-domain-files*
'( "heath:>falken>qp>domains>steam-plant>physob-cs"
 "heath:>falken>qp>domains>steam-plant>thermo-objects"
 "heath:>falken>qp>domains>steam-plant>sources-sinks"
 "heath:>falken>qp>domains>steam-plant>level1"
 "heath:>falken>qp>domains>steam-plant>level4"
 "heath:>falken>qp>domains>steam-plant>level5"
 "heath:>falken>qp>domains>steam-plant>operating-assumptions"))