;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: ADB; -*-

(in-package 'adb)

(proclaim '(special *dds-answers*))

;;;; Simple incremental dependency-directed search abstraction

(defstruct (dds-state (:type list))
  status				    ; :INITIAL, :ON, :EMPTY
  choice-sets)				    ; list of choice sets

(defstruct (choice-set (:type list))
  choices				    ; list of choices
  queue)				    ; current queue of choices

(defun dds-exhaustive (generator) ; runs generator until exhausted
  (setq *dds-answers* nil)
  (do ((answer (try-next generator) (try-next generator)))
      ((null answer) (length *dds-answers*))
    (if answer (push answer *dds-answers*))))

(defun try-next (dds)
  ;; Goes through the choices remaining in the dds state,
  ;; returning the first one which
  ;; does not lead to a contradiction.
  (do ((choices (get-next dds)
		(get-next dds))
       (focus nil))
      ((eq choices ':EMPTY))
    (setq focus (create-environment choices))
    (when (and focus (not (tms::env-contradictory focus)))
        (change-focus focus) ;; reset focus mechanism
        (run) ;; look for bugs
     (if (not (tms::env-contradictory focus)) (return focus)))))

(defun get-next (dds)
  (restore-choice-sets dds)
  (cond ((eq (dds-state-status dds) ':EMPTY) ':EMPTY)
	(t (mapcar #'(lambda (choice-set)
		       (car (choice-set-queue choice-set)))
		   (dds-state-choice-sets dds)))))

;;;; More DDS routines

(defun restore-choice-sets (dds)
  (case (dds-state-status dds)
    (:INITIAL (setf (dds-state-status dds) ':ON))
    (:EMPTY ':EMPTY)
    (t (restore-queued-choices (dds-state-choice-sets dds) dds))))

(defun restore-queued-choices (CS-queues dds-state)
  ;; takes list of queues for choices, restoring elements as necessary
  (cond ((null CS-queues) (setf (dds-state-status dds-state) ':EMPTY))
	((null (cdr (choice-set-queue (car CS-queues))))
	 ;; no more choices left in first one, so restore and decrement next one.
	 (setf (choice-set-queue (car CS-queues))
	       (choice-set-choices (car CS-queues)))
	 (restore-queued-choices (cdr CS-queues) dds-state))
	(t ;; Just decrement the first one.
	 (pop (choice-set-queue (car CS-queues))))))

(defun initialize-dds-state (dds)
  (dolist (choice-set (dds-state-choice-sets dds))
    (setf (choice-set-queue choice-set) (choice-set-choices choice-set)))
  (setf (dds-state-status dds) ':INITIAL))

 