;;; -*- Mode: LISP; Syntax: Common-Lisp; Package: ADB; -*-

(in-package 'adb)

;;; ATMoSphere Structure Definitions  

(export '(assertion? lisp-form truth-node assertion-class mark assert-plist
	  class-name class-instances class-rules class-tms-node class-plist
	  *assert* *class*))

(defvar *assert* 0) ;; unique id counters
(defvar *class* 0) 

(defvar *new* nil) ;; For experimenting with ATMS clause system

(defstruct (assert #+Symbolics (:type vector)
		   #+Symbolics :named
		   (:conc-name nil)
		   (:print-function print-assert)
		   (:predicate assertion?))
  unique
  lisp-form                                     ;The lisp form of the assertion
  truth-node					;A TMS node
  assertion-class				;The class of it
  mark						;Used for various algorithms
  (assert-plist nil))	                        ;Property list for assertion

(defun print-assert (assert stream depth)
  depth ;; outfox symbolics compiler
  (format stream "#<Assert ~A>#" (lisp-form assert))) ;; need shorter form, really.

(defstruct (class #+Symbolics (:type vector)
		  #+Symbolics :named
		  (:print-function print-class)
		  (:predicate class?))			
  unique
  name						;Symbol
  (instances nil)				;Data with that car (sort of)
  (rules nil)					;Associated rules
  tms-node					;Maps to tms:class object
  (class-plist nil))				;Property list

(defun print-class (class stream depth)
  depth						; What am I supposed to do with this?
  (format stream "#<CLASS ~A>#" (class-name class)))

;; At some point may want to re-introduce unique id's for asserts and classes
;; for shorter printing.  Right now no algorithms need them so maybe don't bother.

;;;; Rule instances


(defun print-runk (runk stream depth)
  (declare (ignore depth))
  (format stream "<Runk ~A ~A ~A ~A>" (runk-matcher runk)
	  (runk-args runk)
	  (runk-in-nodes runk)
	  (runk-imp-nodes runk)))


(defstruct (runk ;(:type list)
		 #+Symbolics :named
		 (:print-function print-runk)
		 (:predicate runk?))
  (matcher nil)
  (body nil)
  (class nil)
  (args nil)
  (in-nodes nil)
  (imp-nodes nil)
  (rest nil))  ;; Pointer to rest of the queue (if any).