;;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: USER -*- 

;;;; ATMoSphere -- a Rule-oriented ATMS inference engine.

;; Copyright (c) 1988, Kenneth D. Forbus, University of Illinois
;; All rights reserved.

;;;; THINGS TO DO
;;; -- Look at the clause stuff.  Maybe it can be simplified.
;;; -- Turn ADB into something that is re-entrant?
;;; ***** Add debug test to see if any asserts have equal lisp forms 
;;;       (somebody side-effected the cons cells in a lisp-form field)
;;; ***** Add better printing for generic environments.

;;; WARNING: When editing this on a symbolics machine, set
;;; the package of the buffer to ADB!

#+Symbolics
(defpackage ADB (:use Common-Lisp-Global))
#+Symbolics
(in-package 'ADB)
#-Symbolics
(in-package 'ADB :use '(Lisp))

(defvar adb::*version-number* "3.2 Beta")

;; In this Post-Symbolics world, we revert to
;; the awful days of the mid-70's.  SIGH!

(defvar *user-package*
	#+:SYMBOLICS 'cl-user
	#+:RT 'user)

(defvar *lisp-package*
	#+:Symbolics 'common-lisp
	#+:RT 'lisp)

;; Portable version

;; Now, if only we had logical pathnames...

(defvar *adb-pathname*
	#+KDF-RT "/u/adb/v3-2/"
	#+QRG-RT "/u/forbus/code/adb/"
	#+UIUC "rube:>adb>v3-2>"
        #+JOHAN "calvin:>adb>cl>"
	#+PARC "bd:>adb>cl>"
	)

(export '(view-assumption lprinc-string lprint view-node))

(pcl::defsystem ADB *adb-pathname*
 ;;; <name> <pre-loaded for loading> <pre-loaded for compiling> <recompile if changed>
  ((symbols () () ())
   (add-ons (symbols) (symbols) (symbols))
   (defs (add-ons) (add-ons) ())
   (lprint (defs) () ())
   (utils (lprint) (defs lprint add-ons)
	  (defs lprint add-ons))
   (top (utils) (defs utils) (defs utils))
   (db (top) (defs utils top) (defs utils top))
   (assert (db) (defs utils top db) (defs utils db))
   (vars (assert) (defs utils) (defs utils))
   (fetch (vars) (defs vars utils) (defs vars utils))
   (clause (fetch) (fetch) (fetch))
   (rules (fetch) (defs utils vars fetch) (defs utils vars fetch))
   (dds (rules) (defs utils vars rules fetch)  (defs utils vars rules fetch))
   (interp (dds) (defs utils vars rules fetch add-ons)
	   (defs utils vars rules fetch add-ons))
   (user (interp) (interp) (interp))
   ;;****** You are here in the revision process
   (close (user) (user) (user))
   (sets (close) (close) (close))
   (quantif (sets) (sets) (sets))
   (nqueens (interp) (interp) (interp))
   (debug (nqueens quantif) (nqueens quantif) (nqueens quantif))))

(defvar *atms-loaded* nil)

(defun load-adb ()
  (unless *atms-loaded* 
	  (tms::load-atms)
	  (setq *atms-loaded* t))
  (pcl::operate-on-system 'adb :Load))

(defun compile-adb ()
  (unless *atms-loaded* 
	  (tms::load-atms)
	  (setq *atms-loaded* t))
  (pcl::operate-on-system 'adb :compile))

(defun recompile-adb ()
  (unless *atms-loaded* 
	  (tms::load-atms)
	  (setq *atms-loaded* t))
  (pcl::operate-on-system 'adb :recompile))

