;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: ADB; -*- 

(in-package 'adb)

;;;; Database retrieval
;
;;Retrieval is simple since we are always indexing by the CAR.
;;
;;***** Maybe the deffast-fetch stuff should be ported at some later time...

(export '(fetch print-status print-status1 unify unify1 make-substitutions))

(defun fetch (pattern &key (condition nil) (bindings nil) (return-assertion? nil)
		      (implied-by nil)
	      &aux class candidates winners result test-type)
  (cond ((not (listp pattern))
	 (error "~%~A isn't a good pattern -- FETCH" pattern))
	((and (variable? (car pattern))
	      (not (boundp (car pattern))))
	 (error "~%~A is not a constant -- FETCH" (car pattern)))
	(t ;; Figure out the condition argument
	 (cond ((tms::envp condition) (setq test-type 'Consistent))
	       ((or (eq condition ':IN)
		    (eq condition ':OUT)) (setq test-type 'EQUAL-STATUS))
	       ((tms::envp implied-by)
		(setq test-type 'implication condition implied-by))
	       (t (setq test-type nil)))
	 (setq class (get-assertion-class pattern))
	 (if class (setq class (lookup-class class)))
	 (setq candidates nil winners nil)
	 (when class (setq candidates (class-instances class)))
	 (dolist (candidate candidates)
	   (when (and (listp (lisp-form candidate))
		      (not (eq (setq result (unify pattern (lisp-form candidate)))
			       ':lossage)))
	     (when (case test-type ;; Type of test is satisfied.
		     (EQUAL-STATUS
		       (if (eq condition ':IN)
			   (tms::in?  (truth-node candidate))
			   (tms::out? (truth-node candidate))))
		     (IMPLICATION (tms::true-in? (truth-node candidate)
						 condition))
		     (Consistent (tms::consistent-in? (truth-node candidate)
						      condition)) 
		     (T t))
	       (cond (bindings ;don't bother with error checking
		      (push (mapcar #'(lambda (var)
					(cdr (assoc var result :test #'equal)))
				    bindings)
			    winners))
		     (return-assertion? (push candidate winners))
		     (t (push (lisp-form candidate) winners))))))
	 winners)))

;;;; Printing matching facts

(defmacro print-status (pattern &optional (status nil))
  `(print-status1 ',pattern ,status))

(defvar *use-lprint-internally?* t)

(defun print-status1 (pattern status &aux facts)
  (setq facts (fetch pattern :condition status))
  (format t "~% ~A matches." (length facts))
  (dolist (f facts)
    (if *use-lprint-internally?* (lprint f)
	(format t "~%~A" f))
    (unless status (format t " ~A" (if (tms::in? (get-tms-node (referent f)))
				       "IN" "OUT"))))
  pattern)

;;;; Unification

(defun unify (a b &optional (bindings nil)) 
  ;;; Question: is this faster than seperate functions?  If interlisp-style
  ;;; block compilation is used it should be, but are the implementors that
  ;;; clever??
  (labels ((unify-match (a b s)
	     (cond ((equal a b) s)
		   ((variable? a) (unify-variable a b s))
		   ((variable? b) (unify-variable b a s))
		   ((or (atom a) (atom b)) ':lossage)
		   ((not (eq ':lossage 
			     (setq s (unify-match (car a) (car b) s))))
		    (unify-match (cdr a) (cdr b) s))
		   (t ':lossage)))
	   (unify-variable (var exp s)
	     (cond ((equal (make-substitutions var s)
			   (make-substitutions exp s)) s)
		   (t (let ((entry (assoc var s :test #'eq)))
			(cond (entry (unify-match (cdr entry)
						  exp
						  s))
			      ((free-in var exp s)
			       (cons (cons var exp) s))
			      (t ':lossage))))))
	   (free-in (var exp s)
	     (cond ((equal var exp) nil)
		   ((variable? exp)
		    (free-in var (cdr (assoc exp s :test #'eq)) s))
		   ((atom exp) t)
		   (t (and (free-in var (car exp) s)
			   (free-in var (cdr exp) s))))))
    (unify-match a b bindings)))

(defun make-substitutions (pattern s)
(cond ((variable? pattern) 
       (or (cdr (assoc pattern s :test #'eq))
	   pattern)) ;use binding if known, otherwise pass it on.
      ((atom pattern) pattern)
      (t (cons (make-substitutions (car pattern) s)
	       (make-substitutions (cdr pattern) s)))))
