;;; -*- Mode: LISP; Syntax: Common-lisp; Package: ADB -*-

(in-package 'ADB)

;;;; Simple quantifier system

;;;This could get complicated really, really fast.
;;; Syntax:

;; (EVERY <var> <set> <expression in var>)
;; where <expression in var> must be something that can be handled by
;;        the current desugarer.
;; The intended meaning is that this expression is true just in case
;; <expression> holds for each member of the set, and false otherwise.
;;  (SOME <var> <set> <expression in var>)
;; Similar, but true if <expression> holds for any element of <set>.

(store-in-packet :ATMoSphere-Basics

(adb::rule :intern (((EVERY ?var ?set ?expr) . :TRUE) :var ?quant
		    ((?set MEMBERS ?elements) . :TRUE) :var ?membership)
	   (format t "~% Triggering on (EVERY ~A ~A ~A) and ~%     (~A MEMBERS ~A)"
		   ?var ?set ?expr ?set ?elements)
 (rlet ((?fquant (:EVAL (cons (car ?quant) :FALSE)))
	(?exprs (:EVAL (mapcar #'(lambda (val)
				 (subst val ?var ?expr)) ?elements)))) 
       ;; Positive justification
       (justify ?quant
		`(,?membership  ,@ ?exprs) :EVERY-DEFINITION)
       ;; Negative justifications
       (dolist (val ?exprs)
	 (justify ?fquant `(,?membership ,(flip-truths val)) :EVERY-DEFINITION))
       ;; And, of course, we need to go backwards
       (dolist (val ?exprs)
	 (justify val `(,?membership ,?quant) :EVERY-DEFINITION))
       (nogood `(,?membership ,?fquant ,@ ?exprs)
	       :EVERY-DEFINITION))))

(store-in-packet :ATMoSphere-Basics
 (adb::rule :intern (((SOME ?var ?set ?expr) . :TRUE) :var ?quant
		    ((?set MEMBERS ?elements) . :TRUE) :var ?membership)
 (rlet ((?fquant (:EVAL (cons (car ?quant) :FALSE)))
	(?exprs (:EVAL (mapcar #'(lambda (val)
				 (subst val ?var ?expr)) ?elements)))) 
       ;; Negative justification
       (justify ?fquant
		`(,?membership  ,@ (mapcar #'flip-truths ?exprs)) :SOME-DEFINITION)
       ;; Positive justifications
       (dolist (val ?exprs)
	 (justify ?quant `(,?membership ,val) :SOME-DEFINITION))
       ;; And, of course, we need to go backwards
       (dolist (val ?exprs)
	 (justify (flip-truths val) `(,?membership ,?fquant) :SOME-DEFINITION))
       (nogood `(,?membership ,?quant ,@ (mapcar #'flip-truths ?exprs))
	       :SOME-DEFINITION))))
